/**
 * JASMINe
 * Copyright (C) 2010 Bull S.A.S.
 * Copyright (C) 2010 France Telecom
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.monitoring.view
{
    import com.adobe.cairngorm.control.CairngormEventDispatcher;

    import flash.events.Event;
    import flash.events.TimerEvent;
    import flash.net.FileFilter;
    import flash.utils.Dictionary;
    import flash.utils.Timer;

    import mx.charts.CategoryAxis;
    import mx.charts.GridLines;
    import mx.charts.Legend;
    import mx.charts.LineChart;
    import mx.charts.chartClasses.Series;
    import mx.charts.series.AreaSeries;
    import mx.charts.series.ColumnSeries;
    import mx.charts.series.LineSeries;
    import mx.collections.ArrayCollection;
    import mx.containers.FormHeading;
    import mx.containers.FormItem;
    import mx.containers.TitleWindow;
    import mx.controls.*;
    import mx.events.*;
    import mx.graphics.Stroke;
    import mx.managers.CursorManager;
    import mx.managers.PopUpManager;
    import mx.messaging.events.MessageEvent;
    import mx.messaging.messages.AsyncMessage;
    import mx.rpc.events.FaultEvent;

    import org.ow2.jasmine.kerneos.common.event.KerneosNotificationEvent;
    import org.ow2.jasmine.monitoring.eos.common.controls.DateTimePicker;
    import org.ow2.jasmine.monitoring.eos.common.controls.ServerSideExceptionWindow;
    import org.ow2.jasmine.monitoring.eos.common.events.LayoutChangeEvent;
    import org.ow2.jasmine.monitoring.eos.common.events.LoadEvent;
    import org.ow2.jasmine.monitoring.eos.common.events.PodStateChangeEvent;
    import org.ow2.jasmine.monitoring.eos.common.events.ServerSideExceptionEvent;
    import org.ow2.jasmine.monitoring.eos.common.managers.PodLayoutManager;
    import org.ow2.jasmine.monitoring.eos.common.managers.StateManager;
    import org.ow2.jasmine.monitoring.eos.common.view.AreaChartBase;
    import org.ow2.jasmine.monitoring.eos.common.view.ColumnChartBase;
    import org.ow2.jasmine.monitoring.eos.common.view.LineChartBase;
    import org.ow2.jasmine.monitoring.eos.common.view.LoadWindow;
    import org.ow2.jasmine.monitoring.eos.common.view.Pod;
    import org.ow2.jasmine.monitoring.eos.monitoring.event.*;
    import org.ow2.jasmine.monitoring.eos.monitoring.model.*;
    import org.ow2.jasmine.monitoring.eos.monitoring.vo.*;

    /**
     * This is the action script part of the Monitoring EoS module
     */
    [Bindable]
    public class MonitoringUtil
    {
        // Keep a reference on the main application (mxml part)
        private var main:Object;

        private var model:MonitoringModelLocator=MonitoringModelLocator.getInstance();
        private var dispatcher:CairngormEventDispatcher=CairngormEventDispatcher.getInstance();

        private var podLayoutManager:PodLayoutManager=null;

        // series informations: [id] -> Serie
        private var seriesInfo:Dictionary=null;

        // series data providers: [id] -> ArrayCollection
        private var seriesDataProviders:Dictionary=null;

        // graphs: [id] -> LineChart
        private var graphs:Dictionary=null;

        private var changeGraphButton:ComboBox=null;

        //storage of the graphs and the pods
        private var graphsTable:Dictionary=null;

        // for each graph, give the list of series
        // [graph.id] -> Array[serie.id]
        // needed when changing types of graphs
        private var dataProviderTable:Dictionary=null;

        // type of the graph (area, line, column)
        private static const LINE_CHART:int=0;
        private static const AREA_CHART:int=1;
        private static const COLUMN_CHART:int=2;
        private var graphDispo:Object=[{label: "Line Graph"}, {label: "Area Graph"}, {label: "Column Graph"}];

        public var dataSelectMode:ArrayCollection=new ArrayCollection(["auto data select", "manual data select"]);

        // Must be public because accessed from PopUp windows
        public var shownDataPercentage:int=100;
        public var intelligentSelection:Boolean=true;

        private var monitoringTimer:Timer=null;

        public var timerLength:Number=5;

        public var modeNames:ArrayCollection=new ArrayCollection(["live monitoring", "replay from server file", "replay from database", "replay from local file"]);


        /**
         * Start icon (18 pixels)
         */
        [Embed(source='/../assets/start18.png')]
        [Bindable]
        public var startIcon18:Class;

        /**
         * Stop icon (18 pixels)
         */
        [Embed(source='/../assets/stop18.png')]
        [Bindable]
        public var stopIcon18:Class;

        public function MonitoringUtil(mainApp:Object)
        {
            this.main=mainApp;
        }

        public function initComponent():void
        {
            model.util=this; // used to retrive this from every module

            // Listen to server side events
            dispatcher.addEventListener(ServerSideExceptionEvent.SERVER_SIDE_EXCEPTION, ServerSideExceptionEvent.show);

            // First thing to do
            loadConfiguration();
        }

        private function flushLists():void
        {
            this.seriesInfo=new Dictionary();
            this.seriesDataProviders=new Dictionary();
            this.graphs=new Dictionary();
            this.graphsTable=new Dictionary();
            this.dataProviderTable=new Dictionary();
        }

        private function flushProviders():void
        {
            for each (var provider:ArrayCollection in this.seriesDataProviders)
            {
                // provider.removeAll() doesn't work very well and is too costly
                provider.source=new Array();
            }
        }

        /**
         * Error from Server
         */
        public function set error(event:FaultEvent):void
        {
            if (this.monitoringTimer != null)
            {
                this.monitoringTimer.stop();
            }
            main.startStopMonitoringButton.enabled=true;
            main.replayButton.enabled=false;
            main.replayServerButton.enabled=false;
            main.loadConfButton.enabled=true;

            Alert.show("EoS has been disconnected", "Error occured");
        }

        // -----------------------------------------------------------------------------
        // Mode management
        // -----------------------------------------------------------------------------

        public function changeMode(event:DropdownEvent):void
        {
            // Set currentState
            switch (main.mode.selectedIndex)
            {
                case 0:
                    // live monitoring
                    main.currentState="liveMonitoring";
                    break;
                case 1:
                    // replay from server file
                    var replayWindow:ReplayWindow=PopUpManager.createPopUp(main.graphsCanvas, ReplayWindow, true) as ReplayWindow;
                    replayWindow.addEventListener(LoadEvent.LOADED, loadDatas);
                    replayWindow.init(this);
                    PopUpManager.centerPopUp(replayWindow);
                    main.currentState="replayFile";
                    break;
                case 2:
                    // replay from database
                    var dbWindow:DatabaseWindow=PopUpManager.createPopUp(main.graphsCanvas, DatabaseWindow, true) as DatabaseWindow;
                    dbWindow.init(this);
                    PopUpManager.centerPopUp(dbWindow);
                    main.currentState="replayDataBase";
                    break;
                case 3:
                    // replay from local file
                    var loadWindows:LoadWindow=PopUpManager.createPopUp(main.graphsCanvas, LoadWindow, true) as LoadWindow;
                    var xmlTypes:FileFilter=new FileFilter("Log files (*.log, *.txt, *csv)", "*.log;*.txt;*.csv");
                    loadWindows.fileFilter=new Array(xmlTypes);
                    loadWindows.title="Load log file to replay ...";
                    loadWindows.addEventListener(LoadEvent.LOADED, loadDatas);
                    PopUpManager.centerPopUp(loadWindows);
                    main.currentState="replayFile";
                    break;
            }
        }

        /**
         * Enter liveMonitoring state
         */
        public function liveMonitoring():void
        {
        }

        /**
         * Enter replayDataBase state
         */
        public function replayDataBase():void
        {
        }

        /**
         * Enter replayFile state
         */
        public function replayFile():void
        {
        }

        /**
         * Fill in the providers with datas.
         * Used for both replay and live monitoring
         */
        public function set dataProviders(prov:ArrayCollection):void
        {
            if (main.currentState != "liveMonitoring")
            {
                replayDataLoad(prov);
            }
            else
            {
                liveDataLoad(prov);
            }
        }

        /**
         * Replay, from file or from database
         */
        private function replayDataLoad(provider:ArrayCollection):void
        {
            // end of data loading
            hideProgressBar();

            //threshold value to determine the points number to keep on each graph
            var thresholdValue:int=1200;

            for each (var i:Object in this.graphs)
            {
                i.setMonitoringStatus("loaded");
            }

            // add each value of the provider to the right serie
            for each (var sr:SerieResult in provider)
            {
                var evt:EventDataVO=null;
                if (sr.results.length > thresholdValue)
                {
                    if (this.intelligentSelection)
                    {
                        // Compute automatically the value for shownDataPercentage
                        this.shownDataPercentage=int(thresholdValue * 100 / sr.results.length);
                        if (this.shownDataPercentage == 0)
                        {
                            this.shownDataPercentage=1;
                        }
                    }
                    var previousValue:int=0;
                    var j:int=0;
                    for each (evt in sr.results)
                    {
                        j++;
                        if (previousValue < int(this.shownDataPercentage * j / 100))
                        {
                            previousValue++;
                            var item:Object={date: time2date(evt.date), value: evt.value};
                            (this.seriesDataProviders[sr.id] as ArrayCollection).addItem(item);
                        }
                    }
                }
                else
                {
                    for each (evt in sr.results)
                    {
                        var item:Object={date: time2date(evt.date), value: evt.value};
                        (this.seriesDataProviders[sr.id] as ArrayCollection).addItem(item);
                    }
                }
            }
        }

        /**
         * Live monitoring only
         */
        private function liveDataLoad(provider:ArrayCollection):void
        {
            for each (var sr:SerieResult in provider)
            {
                for each (var evt:EventDataVO in sr.results)
                {
                    var item:Object={date: time2date(evt.date), value: evt.value};
                    (this.seriesDataProviders[sr.id] as ArrayCollection).addItem(item);
                }
            }
        }

        /**
         * Convert a timestamp from String to Date
         * @param time Date as a String
         * @return date as a Date
         */
        private function time2date(time:String):Date
        {
            var time1:Array=time.split(" ");

            var time2:Array=time1[0].split("/");
            var year:Number=new Number(time2[0]);
            var month:Number=new Number(time2[1]) - 1;
            var day:Number=new Number(time2[2]);

            var time3:Array=time1[1].split(":");
            var hour:Number=new Number(time3[0]);
            var min:Number=new Number(time3[1]);
            var sec:Number=new Number(time3[2]);

            var date:Date=new Date(year, month, day, hour, min, sec);
            return date;
        }

        // -----------------------------------------------------------------------------
        // Progress Bar
        // -----------------------------------------------------------------------------

        /**
         * Title window containing the progress Bar.
         */
        private var titleWindow:TitleWindow=null;
        private var progressBar:ProgressBar=null;

        /**
         * Show a progress bar.
         */
        private function showProgressBar(text:String):void
        {
            hideProgressBar();

            // create the progress bar.
            progressBar=new ProgressBar();
            progressBar.indeterminate=true;
            progressBar.percentWidth=100;
            progressBar.label=text;

            // create the popup.
            titleWindow=PopUpManager.createPopUp(main.graphsCanvas, TitleWindow, true) as TitleWindow;
            titleWindow.title="Please wait ...";
            titleWindow.width=main.width / 4;
            // add the progress bar to the popup
            titleWindow.addChild(progressBar);
            // center the popup.
            PopUpManager.centerPopUp(titleWindow);
        }

        /**
         * Hide the progress Bar.
         */
        private function hideProgressBar():void
        {
            if (titleWindow != null)
            {
                // remove the popup from the screen.
                PopUpManager.removePopUp(titleWindow);
                titleWindow=null;
                progressBar=null;
            }
        }

        // -----------------------------------------------------------------------------
        // Load Configuration File
        // -----------------------------------------------------------------------------

        // popup for config selection
        private var configWindow:ConfigWindow=null;

        // popup for editor
        private var editConfigWindow:EditConfigWindow=null;

        // Name of the file
        public var configFileName:String;

        // params found in the configuration file
        public var params:ArrayCollection=null;

        // For each param, give its text field with the chosen value.
        private var paramValues:Dictionary=new Dictionary();

        // Config file splitted in an Array of Strings
        // Each item is either plain text or param name.
        public var fileTokens:ArrayCollection=null;

        // Config File, with params replaced by their value
        public var strConfigFile:String="";

        /**
         * Get the list of configuration files
         */
        public function getFileList():void
        {
            var ev:MonitoringEvent;
            ev=new MonitoringEvent(MonitoringEvent.GET_CONFIG_FILENAMES_EVENT + model.componentID);
            ev.args="*.xml";
            CairngormEventDispatcher.getInstance().dispatchEvent(ev);
        }

        public function loadConfiguration():void
        {
            configWindow=PopUpManager.createPopUp(main.graphsCanvas, ConfigWindow, true) as ConfigWindow;
            configWindow.width=main.graphsCanvas.width;
            configWindow.height=main.graphsCanvas.height;
            configWindow.init(this);
            PopUpManager.centerPopUp(configWindow);
        }

        public function uploadConfig():void
        {
            var loadWindows:LoadWindow=PopUpManager.createPopUp(configWindow, LoadWindow, true) as LoadWindow;
            var xmlTypes:FileFilter=new FileFilter("XML files", "*.xml");
            loadWindows.fileFilter=new Array(xmlTypes);
            loadWindows.addEventListener(LoadEvent.LOADED, loadEnd);
            PopUpManager.centerPopUp(loadWindows);
        }

        public function loadEnd(event:LoadEvent):void
        {
            // rebuild file list with the uploaded file added to the list.
            getFileList();
        }

        /**
         * A file has been selected in the List:
         * Load the file from server.
         */
        public function configFileLoad(path:String, name:String):void
        {
            // load the configuration file
            var ev:MonitoringEvent=new MonitoringEvent(MonitoringEvent.LOAD_CONF_EVENT + model.componentID);
            ev.loadFile=path;
            dispatcher.dispatchEvent(ev);

            configFileName=name;

            // Display only the last part of the full path
            main.confFile.text=name;

            // default mode is live monitoring
            main.currentState="liveMonitoring";
        }

        /**
         * A new config file has been read from server
         * This is triggered by the binding when model has changed,
         * usually when user has selected a config file in the list.
         * @param file String form of the config file
         */
        public function set strconf(file:String):void
        {
            // Look for params the config file
            lookForParams(file);
            CursorManager.removeBusyCursor();
            if (this.params != null)
            {
                configWindow.currentState="params";
            }
            else
            {
                this.strConfigFile=this.fileTokens.source[0];
                configWindow.currentState="selected";
            }
        }

        /**
         * Read the file as a String, and look for params.
         * params should have the form : @paramName@
         * @param file The String form of the config xml file.
         *
         * Build a list of token in this.fileTokens
         * Build a list of params in this.params
         */
        private function lookForParams(file:String):void
        {
            this.params=null;
            this.fileTokens=new ArrayCollection();
            var tokens:Array=file.split("@");
            var paramfound:Boolean=false;
            for each (var token:String in tokens)
            {
                this.fileTokens.addItem(token);
                if (paramfound)
                {
                    if (token.length < 22)
                    {
                        // init with token value.
                        // will be filled later by the user.
                        if (this.params == null)
                        {
                            this.params=new ArrayCollection();
                        }
                        if (!this.params.contains(token))
                        {
                            this.params.addItem(token);
                        }
                    }
                    else
                    {
                        // Cannot go ahead. Forget parameters
                        Alert.show("Bad configuration file: parameter too long:" + token);
                        this.params=null;
                        this.fileTokens=new ArrayCollection();
                        this.fileTokens.addItem(file);
                        break;
                    }
                }
                paramfound=!paramfound;
            }
        }

        /**
         * Enter state params: params have been found in the file.
         * Populate the Form with each param found.
         */
        public function configParams():void
        {
            // Dynamically build a Form to get the params
            configWindow.formparam.removeAllChildren();
            var formHeading:FormHeading=new FormHeading();
            formHeading.label="Set parameter values (default = keep the original text)";
            configWindow.formparam.addChild(formHeading);
            var first:Boolean=true;
            for each (var param:String in this.params)
            {
                var item:FormItem=new FormItem();
                item.label=param;
                var input:TextInput=new TextInput();
                input.text="@" + param + "@";
                item.addChild(input);
                paramValues[param]=input;
                configWindow.formparam.addChild(item);
                if (first)
                {
                    // Set focus on first param
                    input.setFocus();
                    input.drawFocus(true);
                    first=false;
                }
            }
        }

        /**
         * File has been selected, and params have been replaced.
         * Apply this config file
         */
        public function applyConfig():void
        {
            // remove popUp window
            PopUpManager.removePopUp(configWindow);
            configWindow=null;

            parseXmlFile();
        }

        public function confirmParams():void
        {
            expandParams();
            configWindow.currentState="selected";
        }

        public function fileSelected():void
        {
            // Sometimes, we didn't go in "set strconf"
            CursorManager.removeBusyCursor();
        }

        /**
         * Save the current configuration file
         */
        public function saveConfig():void
        {
            configWindow.currentState="savefile";
        }

        /**
         * Save the current configuration file
         */
        public function saveFile():void
        {
            var filename:String=configWindow.fname.text;
            var ev:MonitoringEvent=new MonitoringEvent(MonitoringEvent.SAVE_CONF_EVENT + model.componentID);
            ev.loadFile=filename;
            ev.args=this.strConfigFile;
            dispatcher.dispatchEvent(ev);

            configWindow.currentState="selected";
        }

        public function saveChanges():void
        {
            var ev:MonitoringEvent=new MonitoringEvent(MonitoringEvent.SAVE_CONF_EVENT + model.componentID);
            ev.loadFile=this.configFileName;
            ev.args=this.strConfigFile;
            dispatcher.dispatchEvent(ev);
        }

        public function removeConfig():void
        {
            var ev:MonitoringEvent=new MonitoringEvent(MonitoringEvent.REMOVE_CONF_EVENT + model.componentID);
            ev.configFile=configWindow.listFiles.selectedItem as ConfigFile;
            dispatcher.dispatchEvent(ev);
        }

        /**
         * Save the current configuration file
         */
        public function renameConfig():void
        {
            configWindow.currentState="renamefile";
        }

        public function renameFile():void
        {
            var filename:String=configWindow.nfname.text;
            var ev:MonitoringEvent=new MonitoringEvent(MonitoringEvent.RENAME_CONF_EVENT + model.componentID);
            ev.configFile=configWindow.listFiles.selectedItem as ConfigFile;
            ev.loadFile=filename;
            dispatcher.dispatchEvent(ev);

            configWindow.currentState="selected";
        }

        /**
         * Build this.strConfigFile
         * must replace params if any
         */
        public function expandParams():void
        {
            // Clean any previous operation
            this.strConfigFile="";

            // Replace params if needed
            for each (var token:String in this.fileTokens)
            {
                if (paramValues[token] != null)
                {
                    // replace by the chosen value
                    this.strConfigFile+=(paramValues[token] as TextInput).text;
                }
                else
                {
                    // do not replace this part
                    this.strConfigFile+=token;
                }
            }
        }

        public function editSource():void
        {
            // Create Editor in a popup window
            editConfigWindow=PopUpManager.createPopUp(main.graphsCanvas, EditConfigWindow, true) as EditConfigWindow;
            editConfigWindow.width=main.graphsCanvas.width;
            editConfigWindow.height=main.graphsCanvas.height;
            editConfigWindow.title=configFileName;
            editConfigWindow.init(this);
            PopUpManager.centerPopUp(editConfigWindow);
        }

        private function parseXmlFile():void
        {
            // Parse the XML file
            var doc:XML=new XML(this.strConfigFile);

            main.graphsCanvas.removeAllChildren();
            this.flushLists();
            this.initPodLayoutManager();

            // for each serie
            for each (var node:XML in doc.child("serie-def"))
            {
                // Build a Serie VO and keep it in seriesInfo
                var serie:Serie=new Serie();
                serie.id=node.@id.toString();
                serie.name=node.name;
                serie.description=node.description;
                if (serie.description == "")
                    serie.description=serie.name;
                serie.ycol=node.y.@col;
                serie.ytype=node.y.@type;
                serie.pattern=node.pattern;
                serie.server=node.server;
                this.seriesInfo[serie.id]=serie;

                // init data provider of the serie
                this.seriesDataProviders[serie.id]=new ArrayCollection();
            }

            // for each graph
            var pod:Pod=null;
            for each (var nodeG:XML in doc.child("graph"))
            {
                var graphId:String=nodeG.@id.toString();

                // series for the graph
                var graphSeriesValue:ArrayCollection=new ArrayCollection();


                // title
                var titleValue:String=nodeG.title;

                //type of chart
                var chartType:String=nodeG.chartType;
                var chartSelected:int;
                if (chartType == "lineChartConst")
                {
                    chartSelected=LINE_CHART;
                }
                if (chartType == "areaChartConst")
                {
                    chartSelected=AREA_CHART;
                }
                if (chartType == "columnChartConst")
                {
                    chartSelected=COLUMN_CHART;
                }

                //create an array to add all the series for the current graph
                var graphSeries:Array=new Array;

                // add each serie for the current graph
                for each (var serieNode:XML in nodeG.child("serie"))
                {
                    var serieId:String=serieNode.@ref.toString();
                    graphSeries.push(serieId);

                    var stroke:Stroke=new Stroke();
                    stroke.weight=1;

                    switch (chartSelected)
                    {
                        case LINE_CHART:
                            var lineSerie:LineSeries=new LineSeries();
                            lineSerie.id=serieId;
                            lineSerie.dataProvider=this.seriesDataProviders[serieId];
                            if (serieNode.toString() != "")
                            {
                                lineSerie.displayName=serieNode.toString();
                            }
                            else
                            {
                                lineSerie.displayName=this.seriesInfo[serieId].description;
                            }
                            lineSerie.yField="value";
                            lineSerie.xField="date";
                            lineSerie.setStyle("lineStroke", stroke);
                            lineSerie.filterData=false;
                            graphSeriesValue.addItem(lineSerie);
                            break;

                        case AREA_CHART:
                            var areaSerie:AreaSeries=new AreaSeries();
                            areaSerie.id=serieId;
                            areaSerie.dataProvider=this.seriesDataProviders[serieId];
                            if (serieNode.toString() != "")
                            {
                                areaSerie.displayName=serieNode.toString();
                            }
                            else
                            {
                                areaSerie.displayName=this.seriesInfo[serieId].description;
                            }
                            areaSerie.yField="value";
                            areaSerie.xField="date";
                            graphSeriesValue.addItem(areaSerie);
                            break;

                        case COLUMN_CHART:
                            var columnSerie:ColumnSeries=new ColumnSeries();
                            columnSerie.id=serieId;
                            columnSerie.dataProvider=this.seriesDataProviders[serieId];
                            if (serieNode.toString() != "")
                            {
                                columnSerie.displayName=serieNode.toString();
                            }
                            else
                            {
                                columnSerie.displayName=this.seriesInfo[serieId].description;
                            }
                            columnSerie.yField="value";
                            columnSerie.xField="date";
                            columnSerie.filterData=false;
                            graphSeriesValue.addItem(columnSerie);
                            break;
                    }

                }
                this.dataProviderTable[graphId]=graphSeries;

                switch (chartSelected)
                {
                    case LINE_CHART:
                        var lineChartBase:LineChartBase=new LineChartBase();
                        lineChartBase.setCharts(graphSeriesValue);
                        this.graphs[graphId]=lineChartBase;
                        break;

                    case AREA_CHART:
                        var areaChartBase:AreaChartBase=new AreaChartBase();
                        areaChartBase.setCharts(graphSeriesValue);
                        this.graphs[graphId]=areaChartBase;
                        break;

                    case COLUMN_CHART:
                        var columnChartBase:ColumnChartBase=new ColumnChartBase();
                        columnChartBase.setCharts(graphSeriesValue);
                        this.graphs[graphId]=columnChartBase;
                        break;
                }

                this.changeGraphButton=this.setChangeGraphButton(graphId);
                this.changeGraphButton.selectedIndex=chartSelected;

                //add the chart and the button to the pod
                pod=new Pod();
                pod.title=titleValue;
                pod.id=graphId;
                pod.addChild(this.graphs[graphId]);
                pod.addChild(this.changeGraphButton);
                this.addPod(pod, false);
                //filling the table for the correspondence between the pods and
                //the graphs
                this.graphsTable[pod.id]=pod;
            }
            this.displayPods(pod);
        }

        // -----------------------------------------------------------------------------
        // Start / Stop live monitoring
        // -----------------------------------------------------------------------------

        private var monitoringStarted:Boolean=false;

        public function toggleMonitoring():void
        {
            if (this.monitoringStarted == false)
            {
                this.startMonitoring();
                main.startStopMonitoringButton.label="Stop";
                main.startStopMonitoringButton.toolTip="Stop live monitoring";
                main.startStopMonitoringButton.setStyle("icon", this.stopIcon18);
                main.loadConfButton.enabled=false;
                main.mode.enabled=false;
                this.monitoringStarted=true;
            }
            else
            {
                this.stopMonitoring();
                main.startStopMonitoringButton.label="Start";
                main.startStopMonitoringButton.toolTip="Start live monitoring";
                main.startStopMonitoringButton.setStyle("icon", this.startIcon18);
                main.loadConfButton.enabled=true;
                main.mode.enabled=true;
                this.monitoringStarted=false;
            }
        }

        /**
         * Start the live monitoring
         */
        public function startMonitoring():void
        {
            flushProviders();
            this.monitoringTimer=new Timer(10000, Infinity);
            this.monitoringTimer.addEventListener(TimerEvent.TIMER, poll);
            this.monitoringTimer.start();
            main.refreshTimeSlider.value=this.timerLength;
            for each (var i:Object in this.graphs)
            {
                i.setMonitoringStatus("live");
            }
        }

        /**
         * Called by timer for live monitoring
         */
        private function poll(event:TimerEvent):void
        {
            var ev:MonitoringEvent=new MonitoringEvent(MonitoringEvent.START_MONITORING + model.componentID);
            ev.series=new ArrayCollection();
            for each (var s:Serie in this.seriesInfo)
            {
                ev.series.addItem(s);
            }
            dispatcher.dispatchEvent(ev);
        }

        /**
         * Set the timer refresh period
         */
        public function timerRefreshPeriodChange():void
        {
            if (main.refreshTimeSlider.value == 0)
            {
                this.monitoringTimer.delay=this.timerLength * 1000;
                main.refreshTimeSlider.value=this.timerLength;
            }
            this.monitoringTimer.delay=main.refreshTimeSlider.value * 1000;
        }

        /**
         * Stop the live monitoring
         */
        public function stopMonitoring():void
        {
            this.monitoringTimer.stop();
            var event:MonitoringEvent=new MonitoringEvent(MonitoringEvent.STOP_MONITORING + model.componentID);
            dispatcher.dispatchEvent(event);

            for each (var i:Object in this.graphs)
            {
                i.setMonitoringStatus("stopped");
            }
        }

        // -----------------------------------------------------------------------------
        // Replay from file
        // -----------------------------------------------------------------------------

        public var replayFileName:String;

        private function loadDatas(event:LoadEvent):void
        {
            this.replayFileName = event.filename;

            // Display only the last part of the full path
            var maxsize:int=32;
            var index:int=this.replayFileName.length - maxsize;
            var troncated:String=this.replayFileName;
            if (index > 0)
                troncated=this.replayFileName.substring(index);
            main.replayFile.text=troncated;

            refreshReplay();
        }

        public function refreshReplay()
        {
            // Start reading file for Replay
            flushProviders();
            showProgressBar("Loading events");
            var ev:MonitoringEvent=new MonitoringEvent(MonitoringEvent.LOAD_LOG_FILE_EVENT + model.componentID);
            ev.loadFile=this.replayFileName;
            ev.series=new ArrayCollection();
            for each (var s:Serie in this.seriesInfo)
            {
                ev.series.addItem(s);
            }
            dispatcher.dispatchEvent(ev);
        }

        // -----------------------------------------------------------------------------
        // replay from Database
        // -----------------------------------------------------------------------------

        public function loadFromDataBase(from:Date, to:Date):void
        {
            var ev:MonitoringEvent=new MonitoringEvent(MonitoringEvent.LOAD_DATA_BASE + model.componentID);
            ev.series=new ArrayCollection();
            for each (var s:Serie in this.seriesInfo)
            {
                ev.series.addItem(s);
            }
            ev.from=from;
            ev.to=to;
            dispatcher.dispatchEvent(ev);

            flushProviders();

            // display dates
            main.fromDate.text=from.toLocaleString();
            main.toDate.text=to.toLocaleString();

            // Start loading datas
            showProgressBar("Loading events");
        }

        // -----------------------------------------------------------------------------
        // Change Graph Type button
        // -----------------------------------------------------------------------------

        private function setChangeGraphButton(id:String):ComboBox
        {
            var comboBox:ComboBox=new ComboBox();
            comboBox.id=id;

            comboBox.dataProvider=this.graphDispo;
            comboBox.addEventListener(flash.events.Event.CHANGE, changeGraphButtonMenuClickHandler);
            return comboBox;
        }

        private function changeGraphButtonMenuClickHandler(event:Event):void
        {
            var graphSeriesValue:ArrayCollection=new ArrayCollection();

            //Get the type of the graph
            var chartType:String=event.currentTarget.value.toString();
            var chartSelected:int;
            if (chartType == "Line Graph")
            {
                chartSelected=LINE_CHART;
            }
            if (chartType == "Area Graph")
            {
                chartSelected=AREA_CHART;
            }
            if (chartType == "Column Graph")
            {
                chartSelected=COLUMN_CHART;
            }
            //create a new type of chart with the same dataProvider
            var currentDataProvider:String=null;
            switch (chartSelected)
            {
                case LINE_CHART:
                    for each (currentDataProvider in this.dataProviderTable[event.currentTarget.id.toString()])
                    {
                        var lineChartBase:LineChartBase=new LineChartBase();
                        var lineSerie:LineSeries=new LineSeries();
                        lineSerie.id=currentDataProvider;
                        lineSerie.dataProvider=this.seriesDataProviders[currentDataProvider];
                        lineSerie.displayName=this.seriesInfo[currentDataProvider].description;
                        lineSerie.yField="value";
                        lineSerie.xField="date";
                        lineSerie.filterData=false;
                        var stroke:Stroke=new Stroke();
                        stroke.weight=1;
                        lineSerie.setStyle("lineStroke", stroke);
                        graphSeriesValue.addItem(lineSerie);
                        lineChartBase.setCharts(graphSeriesValue);
                    }
                    this.graphs[event.currentTarget.id.toString()]=lineChartBase;
                    break;
                case AREA_CHART:
                    for each (currentDataProvider in this.dataProviderTable[event.currentTarget.id.toString()])
                    {
                        var areaChartBase:AreaChartBase=new AreaChartBase();
                        var areaSerie:AreaSeries=new AreaSeries();
                        areaSerie.id=currentDataProvider;
                        areaSerie.dataProvider=this.seriesDataProviders[currentDataProvider];
                        areaSerie.displayName=this.seriesInfo[currentDataProvider].description;
                        areaSerie.yField="value";
                        areaSerie.xField="date";
                        graphSeriesValue.addItem(areaSerie);
                        areaChartBase.setCharts(graphSeriesValue);
                    }
                    this.graphs[event.currentTarget.id.toString()]=areaChartBase;
                    break;
                case COLUMN_CHART:
                    for each (currentDataProvider in this.dataProviderTable[event.currentTarget.id.toString()])
                    {
                        var columnChartBase:ColumnChartBase=new ColumnChartBase();
                        var columnSerie:ColumnSeries=new ColumnSeries();
                        columnSerie.id=currentDataProvider;
                        columnSerie.dataProvider=this.seriesDataProviders[currentDataProvider];
                        columnSerie.displayName=this.seriesInfo[currentDataProvider].description;
                        columnSerie.yField="value";
                        columnSerie.xField="date";
                        columnSerie.filterData=false;
                        graphSeriesValue.addItem(columnSerie);
                        columnChartBase.setCharts(graphSeriesValue);
                    }
                    this.graphs[event.currentTarget.id.toString()]=columnChartBase;
                    break;
            }

            this.changeGraphButton=this.setChangeGraphButton(event.currentTarget.id.toString());
            this.changeGraphButton.selectedIndex=chartSelected;
            //Remove the old graph by using the id of the event
            var title:String=this.graphsTable[event.currentTarget.id.toString()].title;
            main.graphsCanvas.removeChild(this.graphsTable[event.currentTarget.id.toString()]);
            //put the new graph in the good pod
            var pod:Pod=new Pod();
            pod.title=title;
            pod.id=event.currentTarget.id.toString();
            pod.addChild(this.graphs[event.currentTarget.id.toString()]);
            pod.addChild(this.changeGraphButton);
            this.addPod(pod, true);
            this.graphsTable[pod.id]=pod;
            for each (var i:Object in this.graphs)
            {
                if (main.currentState == "liveMonitoring")
                {
                    i.setMonitoringStatus("live");
                }
                else
                {
                    i.setMonitoringStatus("loaded");
                }
            }
        }

        // -----------------------------------------------------------------------------
        // Pod management
        // -----------------------------------------------------------------------------

        /**
         * init the podLayoutManager
         */
        private function initPodLayoutManager():void
        {
            this.podLayoutManager=new PodLayoutManager();
            this.podLayoutManager.container=main.graphsCanvas;
            this.podLayoutManager.id="unique";
            this.podLayoutManager.addEventListener(LayoutChangeEvent.UPDATE, StateManager.setPodLayout);
        }

        /**
         * Add a Pod
         */
        private function addPod(pod:Pod, display:Boolean):void
        {
            var podLen:Number=0;
            for each (var i:Object in this.graphs)
            {
                podLen++;
            }
            var index:Number=StateManager.getPodIndex(this.podLayoutManager.id, pod.id);

            // If the index hasn't been saved move the pod to the last position.
            if (index == -1)
            {
                index=podLen;
            }

            this.podLayoutManager.addItemAt(pod, index, StateManager.isPodMaximized(this.podLayoutManager.id, pod.id));

            if (display)
            {
                displayPods(pod);
            }
        }

        private function displayPods(pod:Pod):void
        {
            pod.addEventListener(FlexEvent.UPDATE_COMPLETE, onCreationCompletePod);
        }

        /**
         * Pod has been created so update the respective PodLayoutManager.
         */
        private function onCreationCompletePod(e:FlexEvent):void
        {
            e.currentTarget.removeEventListener(FlexEvent.UPDATE_COMPLETE, onCreationCompletePod);
            this.podLayoutManager.removeNullItems();
            this.podLayoutManager.updateLayout(false);
            this.podLayoutManager.dispatchEvent(new LayoutChangeEvent(LayoutChangeEvent.UPDATE));
        }

    }
}
